using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;
using System.Threading;

class State
{
	public const int BufferSize = 256;

	public State()
	{
		Buffer = new byte[BufferSize];
	}

	public Socket Client { get; set; }
	public byte[] Buffer { get; set; }
}

class StringState : State
{
	public StringState() : base()
	{
		Data = new StringBuilder();
	}
	
	public StringBuilder Data { get; set; }
}

class Program
{
	static public ManualResetEvent done = new ManualResetEvent(false);

	static public void AcceptCallback(IAsyncResult ar)
	{
		Socket client = ((Socket)ar.AsyncState).EndAccept(ar);
		done.Set();
		
		Console.WriteLine("Kliens kapcsoldott...");
		
		StringState state = new StringState();
		state.Client = client;
		
		client.BeginReceive(state.Buffer, 0, State.BufferSize,
			SocketFlags.None, Program.ReadCallback, state);
	}

	static public void ReadCallback(IAsyncResult ar)
	{
		StringState state = (StringState)ar.AsyncState;
		
		int length = state.Client.EndReceive(ar);
		
		state.Data.Append(Encoding.ASCII.GetString(state.Buffer, 0, length));
		
		Console.WriteLine("A kliens zenete: {0}",
			state.Data);
			
		state.Client.Close();
	}

	static public void Main(string[] args)
	{
		Socket server = null;
		
		try
		{
			server = new Socket(
				AddressFamily.InterNetwork, 
				SocketType.Stream, 
				ProtocolType.Tcp);
		
			IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(args[0]), int.Parse(args[1]));
		
			server.Bind(endPoint);
			server.Listen(100);
		
			while(true)
			{
				done.Reset();
				
				Console.WriteLine("A szerver kapcsolatra vr...");
				
				server.BeginAccept(Program.AcceptCallback, server);
				
				done.WaitOne();
			}
		}
		catch(Exception e)
		{
			Console.WriteLine(e.Message);
		}
		finally
		{
			server.Close();
		}
	}
}